
From: Kaigai Kohei <kaigai@ak.jp.nec.com>

Signed-off-by: KaiGai, Kohei <kaigai@ak.jp.nec.com>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/include/asm-x86_64/atomic.h |   25 +++++++++++++++++++++++++
 1 files changed, 25 insertions(+)

diff -puN include/asm-x86_64/atomic.h~atomic_inc_return-for-x86_64 include/asm-x86_64/atomic.h
--- 25/include/asm-x86_64/atomic.h~atomic_inc_return-for-x86_64	2004-09-15 22:10:51.274180128 -0700
+++ 25-akpm/include/asm-x86_64/atomic.h	2004-09-15 22:10:51.278179520 -0700
@@ -338,6 +338,31 @@ static __inline__ long atomic64_add_nega
 	return c;
 }
 
+/**
+ * atomic_add_return - add and return
+ * @v: pointer of type atomic_t
+ * @i: integer value to add
+ *
+ * Atomically adds @i to @v and returns @i + @v
+ */
+static __inline__ int atomic_add_return(int i, atomic_t *v)
+{
+	int __i = i;
+	__asm__ __volatile__(
+		LOCK "xaddl %0, %1;"
+		:"=r"(i)
+		:"m"(v->counter), "0"(i));
+	return i + __i;
+}
+
+static __inline__ int atomic_sub_return(int i, atomic_t *v)
+{
+	return atomic_add_return(-i,v);
+}
+
+#define atomic_inc_return(v)  (atomic_add_return(1,v))
+#define atomic_dec_return(v)  (atomic_sub_return(1,v))
+
 /* These are x86-specific, used by some header files */
 #define atomic_clear_mask(mask, addr) \
 __asm__ __volatile__(LOCK "andl %0,%1" \
_
